from .base import Aws
from .schemas import TextractResponse, TextractBlock, TextractBlockType, TextractGeometry, TextractQuery, TextractPolygon, TextractRelationship, TextractRelationshipType, TextractEntityTypes, TextractTextTypeEnum, TextractBoundingBox, TextractSelectionStatus, TextractDocumentMetadata
from .textract import AwsTextract